/**
 * LevelEditor.java
 * 3/30/12
 */

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Scanner;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * Draw rectangles and save them to a file.
 * 
 * @author CPSC170A
 */
public class LevelEditor extends JPanel implements MouseMotionListener,
        ActionListener {

    /** Window dimensions. */
    private static final Dimension WIN_DIM = new Dimension(640, 480);

    /** Panel dimensions. */
    private static final Dimension PANEL_DIM = new Dimension(65536, 1024);

    /** Sprite width and height in pixels. */
    private static final int SPRITE_SIZE = 50;

    /** Window object. */
    private JFrame frame = new JFrame("Level Editor - Count: 0");

    /** List of sprites. */
    private LinkedList<Sprite> sprites = new LinkedList<Sprite>();

    /**
     * Constructor, initializes window.
     * 
     * @throws IOException
     */
    public LevelEditor() {
        JScrollPane scrollPane = new JScrollPane(this);
        scrollPane.setPreferredSize(WIN_DIM);
        setPreferredSize(PANEL_DIM);
        addMouseMotionListener(this);
        initMenu();
        frame.getContentPane().add(scrollPane);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);
    }

    /**
     * Initialize window menu bar.
     */
    public final void initMenu() {
        JMenuBar menuBar = new JMenuBar();
        JMenu fileMenu = new JMenu("File");
        JMenuItem newMenuItem = new JMenuItem("New");
        JMenuItem openMenuItem = new JMenuItem("Open");
        JMenuItem saveMenuItem = new JMenuItem("Save");
        JMenuItem exitMenuItem = new JMenuItem("Exit");
        newMenuItem.addActionListener(this);
        openMenuItem.addActionListener(this);
        saveMenuItem.addActionListener(this);
        exitMenuItem.addActionListener(this);
        fileMenu.add(newMenuItem);
        fileMenu.add(openMenuItem);
        fileMenu.add(saveMenuItem);
        fileMenu.add(exitMenuItem);
        menuBar.add(fileMenu);
        frame.setJMenuBar(menuBar);
        frame.setVisible(true);
    }

    /**
     * Runs the application.
     * 
     * @param args command line argument
     */
    public static void main(final String[] args) {
        new LevelEditor();
    }

    /**
     * Draw all the sprites.
     * 
     * @param g graphics object to draw to
     */
    public final void paintComponent(final Graphics g) {
        super.paintComponent(g);
	for (Sprite currentSprite : sprites) {
	    currentSprite.paint(g);
	}
    }

    /**
     * Add a sprite when the mouse is dragged.
     * 
     * @param event mouse event
     */
    public final void mouseDragged(final MouseEvent event) {
	final int x = event.getX();
	final int y = event.getY();
	Sprite newSprite = new Sprite(x, y, SPRITE_SIZE, SPRITE_SIZE);
	sprites.addLast(newSprite);
        frame.setTitle("Level Editor - Count: " + sprites.size());
        repaint();
    }

    /**
     * Unused in this program.
     * 
     * @param event moust event
     */
    public void mouseMoved(final MouseEvent event) {}

    /**
     * Handle menu selection.
     * 
     * @param event action event
     */
    public final void actionPerformed(final ActionEvent event) {
        String menuText = ((JMenuItem) event.getSource()).getText();
        if (menuText.equals("New")) {
	    sprites = new LinkedList<Sprite>();
            repaint();
        } else if (menuText.equals("Open")) {
            JFileChooser fileChooser = new JFileChooser();
            int response;
            fileChooser.setCurrentDirectory(new File("."));
            response = fileChooser.showOpenDialog(frame);
            if (response == JFileChooser.APPROVE_OPTION) {
		readFile(fileChooser.getSelectedFile());
            }
            repaint();
        } else if (menuText.equals("Save")) {
            JFileChooser fileChooser = new JFileChooser();
            int response;
            fileChooser.setCurrentDirectory(new File("."));
            response = fileChooser.showSaveDialog(frame);
            if (response == JFileChooser.APPROVE_OPTION) {
		writeFile(fileChooser.getSelectedFile());
            }
        } else if (menuText.equals("Exit")) {
            System.exit(0);
        }
    }

    /**
     * Read the sprites in the specified file.
     *
     * @param file text file containing list of sprites
     */
    public final void readFile(File file) {
        LinkedList<Sprite> readSprites = new LinkedList<Sprite>();
        try {
            Scanner scan = new Scanner(file);
	    System.out.println(scan.hasNext());
            double x, y, width, height;
            while (scan.hasNext()) {
                x = scan.nextDouble();
                y = scan.nextDouble();
                width = scan.nextDouble();
                height = scan.nextDouble();
                readSprites.addLast(new Sprite(x, y, width, height));
            }
	    sprites = readSprites;
        } catch (FileNotFoundException e) {
	    JOptionPane.showMessageDialog(frame, "Can not find file: " + file);
        } catch (Exception e) {
	    JOptionPane.showMessageDialog(frame, "Unreadable file: " + file);
	}
    }

    /**
     * Write the currently drawn sprites to the specified file.
     *
     * @param file text file to write the sprites to
     */
    public final void writeFile(File file) {
        PrintWriter out = null;
        try {
            out = new PrintWriter(file);
            for (Sprite currentSprite : sprites) {
                out.println(currentSprite.getX() + " "
			    + currentSprite.getY() + " "
			    + currentSprite.getWidth() + " "
			    + currentSprite.getHeight());
            }
        } catch (IOException e) {
            System.out.println("Unable to write file: " + file);
        } finally {
            if (out != null) {
                out.close();
            }
        }
    }
}
