/** 
 * Sprite.java
 * 3/30/12
 */

import java.awt.Graphics;
import java.awt.Image;
import java.awt.geom.Point2D;

/**
 * Class that represents a 2D sprite.
 * 
 * @author CPSC170A
 */
public class Sprite {
    /** Sprite x coordinate. */
    protected double x;

    /** Sprite y coordinate. */
    protected double y;

    /** Sprite width. */
    protected double width;

    /** Sprite height. */
    protected double height;

    /** Sprite image when drawn. */
    protected Image image;

    /**
     * Initialize a sprite with the specified initial location and size.
     * 
     * @param initX initial x coordinate of upper left corner
     * @param initY initial y coordinate of upper left corner
     * @param initWidth initial width
     * @param initHeight initial height
     */
    public Sprite(final double initX, final double initY,
            final double initWidth, final double initHeight) {
        x = initX;
        y = initY;
        width = initWidth;
        height = initHeight;
        image = null;
    }

    /**
     * Initialize a sprite with the specified initial location, size, and image.
     * 
     * @param initX initial x coordinate of upper left corner
     * @param initY initial y coordinate of upper left corner
     * @param initWidth initial width
     * @param initHeight initial height
     * @param initImage initial image
     */
    public Sprite(final double initX, final double initY,
            final double initWidth, final double initHeight,
            final Image initImage) {
        x = initX;
        y = initY;
        width = initWidth;
        height = initHeight;
        image = initImage;
    }

    /**
     * Draw this sprite based on the viewport location.
     * 
     * @param g graphics object to draw to
     */
    public final void paint(final Graphics g) {
        if (image == null) {
            g.fillRect((int) x, (int) y, (int) width, (int) height);
        } else {
            g.drawImage(image, (int) x, (int) y,
			(int) width, (int) height, null);
        }

    }

    /**
     * Modifies the x or y coordinate of this sprite such that this
     * sprite moves the minimal amount of distance so that this sprite
     * is no longer intersecting with the specified other sprite.
     *
     * @param otherSprite the sprite that this should no longer intersect with
     */
    public void deintersect(final Sprite otherSprite) {
      final double top = topOverlap(otherSprite);
      final double bottom = bottomOverlap(otherSprite);
      final double left = leftOverlap(otherSprite);
      final double right = rightOverlap(otherSprite);
      final double infinity = Double.POSITIVE_INFINITY;
      if (top != infinity && top <= left && top < right) {
        y -= top;
      }
      if (bottom != infinity && bottom < left && bottom <= right) {
        y += bottom;
      }
      if (left != infinity && left < top && left <= bottom) {
        x -= left;
      }
      if (right != infinity && right <= top && right < bottom) {
        x += right;
      }
    }

    /**
     * Returns whether this and the specified other sprite intersect.
     * 
     * @param otherSprite the other sprite to test if this sprite is
     *            intersecting with
     * @return true if this and otherSprite intersect, false otherwise
     */
    public final boolean intersect(final Sprite otherSprite) {
      if (this.getTop() > otherSprite.getBottom()) {
        return false;
      }
      if (this.getBottom() < otherSprite.getTop()) {
        return false;
      }
      if (this.getLeft() > otherSprite.getRight()) {
        return false;
      }
      if (this.getRight() < otherSprite.getLeft()) {
        return false;
      }
      return true;
    }

    /**
     * Returns the amount, in fractional pixels, that this sprite
     * overlaps with the top edge of the specified other sprite.  If
     * this sprite does not overlap with the top of the other sprite
     * then positive infinity is returned.
     * 
     * @param otherSprite the other sprite to compute overlap with
     * @return the amount that this sprite overlaps with the top of otherSprite
     */
    public final double topOverlap(final Sprite otherSprite) {
      if (this.getTop() > otherSprite.getTop()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getBottom() < otherSprite.getTop()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getLeft() > otherSprite.getRight()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getRight() < otherSprite.getLeft()) {
        return Double.POSITIVE_INFINITY;
      }
      return this.getBottom() - otherSprite.getTop();
    }

    /**
     * Returns the amount, in fractional pixels, that this sprite
     * overlaps with the bottom edge of the specified other sprite.  If
     * this sprite does not overlap with the bottom of the other sprite,
     * then positive infinity is returned.
     * 
     * @param otherSprite the other sprite to compute overlap with
     * @return amount that this sprite overlaps with the bottom of otherSprite
     */
    public final double bottomOverlap(final Sprite otherSprite) {
      if (this.getTop() > otherSprite.getBottom()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getBottom() < otherSprite.getBottom()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getLeft() > otherSprite.getRight()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getRight() < otherSprite.getLeft()) {
        return Double.POSITIVE_INFINITY;
      }
      return otherSprite.getBottom() - this.getTop();
    }

    /**
     * Returns the amount, in fractional pixels, that this sprite
     * overlaps with the left edge of the specified other sprite.  If
     * this sprite does not overlap with the left of the other sprite
     * then positive infinity is returned.
     * 
     * @param otherSprite the other sprite to compute overlap with
     * @return the amount that this sprite overlaps with the left of otherSprite
     */
    public final double leftOverlap(final Sprite otherSprite) {
      if (this.getTop() > otherSprite.getBottom()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getBottom() < otherSprite.getTop()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getLeft() > otherSprite.getLeft()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getRight() < otherSprite.getLeft()) {
        return Double.POSITIVE_INFINITY;
      }
      return this.getRight() - otherSprite.getLeft();
    }

    /**
     * Returns the amount, in fractional pixels, that this sprite
     * overlaps with the right edge of the specified other sprite.  If
     * this sprite does not overlap with the right of the other sprite,
     * then positive infinity is returned.
     * 
     * @param otherSprite the other sprite to compute overlap with
     * @return amount that this sprite overlaps with the right of otherSprite
     */
    public final double rightOverlap(final Sprite otherSprite) {
      if (this.getTop() > otherSprite.getBottom()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getBottom() < otherSprite.getTop()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getLeft() > otherSprite.getRight()) {
        return Double.POSITIVE_INFINITY;
      }
      if (this.getRight() < otherSprite.getRight()) {
        return Double.POSITIVE_INFINITY;
      }
      return otherSprite.getRight() - this.getLeft();
    }

    /**
     * @return the coordinate of the top side of this sprite
     */
    public final double getTop() {
        return y;
    }

    /**
     * @return the coordinate of the bottom side of this sprite
     */
    public final double getBottom() {
      return y + height;
    }

    /**
     * @return the coordinate of the left side of this sprite
     */
    public final double getLeft() {
        return x;
    }

    /**
     * @return the coordinate of the right side of this sprite
     */
    public final double getRight() {
        return x + width;
    }

    /**
     * @return the x
     */
    public final double getX() {
        return x;
    }

    /**
     * @param newX the x to set
     */
    public final void setX(final double newX) {
        x = newX;
    }

    /**
     * @return the y
     */
    public final double getY() {
        return y;
    }

    /**
     * @param newY the y to set
     */
    public final void setY(final double newY) {
        y = newY;
    }

    /**
     * @return the width
     */
    public final double getWidth() {
        return width;
    }

    /**
     * @param newWidth the width to set
     */
    public final void setWidth(final double newWidth) {
        width = newWidth;
    }

    /**
     * @return the height
     */
    public final double getHeight() {
        return height;
    }

    /**
     * @param newHeight the height to set
     */
    public final void setHeight(final double newHeight) {
        height = newHeight;
    }

    /**
     * @return the image
     */
    public final Image getImage() {
        return image;
    }

    /**
     * @param newImage the image to set
     */
    public final void setImage(final Image newImage) {
        image = newImage;
    }

    /**
     * @see java.lang.Object#toString()
     * @return string representing this sprite
     */
    public final String toString() {
        return "Sprite [height="
                + height + ", image=" + image + ", width=" + width + ", x=" + x
                + ", y=" + y + "]";
    }

}
